@@ -19,16 +19,18 @@ use rustc_middle::{
       MutatingUseContext, NonMutatingUseContext, NonUseContext, PlaceContext,
       Visitor as MirVisitor,
     },
-    Body, HasLocalDecls, TerminatorKind,
+    Body, FakeReadCause, HasLocalDecls, Statement, StatementKind, Terminator,
+    TerminatorKind,
   },
   ty::TyCtxt,
 };
 use rustc_span::{Span, SpanData};
+use smallvec::{smallvec, SmallVec};
 
 use crate::{
   block_timer,
-  indexed::impls::LocationDomain,
-  mir::utils::{self, SpanDataExt, SpanExt},
+  indexed::impls::{arg_location, LocationDomain},
+  mir::utils::{self, BodyExt, SpanDataExt, SpanExt},
 };
 
 // Collect all the spans for children beneath the visited node.
@@ -187,7 +189,7 @@ impl HirVisitor<'hir> for HirSpanCollector<'_, '_, 'hir, '_> {
 pub struct MirSpannedPlace<'tcx> {
   pub place: mir::Place<'tcx>,
   pub span: SpanData,
-  pub location: mir::Location,
+  pub locations: SmallVec<[mir::Location; 1]>,
 }
 
 struct MirSpanCollector<'a, 'b, 'hir, 'tcx>(&'a mut Spanner<'b, 'hir, 'tcx>);
@@ -200,6 +202,17 @@ impl MirVisitor<'tcx> for MirSpanCollector<'_, '_, '_, 'tcx> {
     location: mir::Location,
   ) {
     trace!("place={place:?} context={context:?} location={location:?}");
+
+    // MIR will sometimes include places assigned to unit, e.g.
+    //   if true { let x = 1; } else { let x = 2; }
+    // then the entire block will have a place with unit value.
+    // To avoid letting that block be selectable, we ignore values with unit type.
+    // This is a hack, but not sure if there's a better way?
+    let body = &self.0.body;
+    if place.ty(body.local_decls(), self.0.tcx).ty.is_unit() {
+      return;
+    }
+
     // Three cases, shown by example:
     //   fn foo(x: i32) {
     //     let y = x + 1;
@@ -210,17 +223,51 @@ impl MirVisitor<'tcx> for MirSpanCollector<'_, '_, '_, 'tcx> {
     // * "x + 1" -- MIR will generate a temporary to assign x into, whose
     //   span is given to "x". That corresponds to MutatingUseContext::Store
     // * "y" -- this corresponds to NonMutatingUseContext::Inspect
-    let span = match context {
-      PlaceContext::MutatingUse(MutatingUseContext::Store)
-      | PlaceContext::NonMutatingUse(NonMutatingUseContext::Inspect) => {
-        let source_info = self.0.body.source_info(location);
-        source_info.span
+    let (span, locations) = match context {
+      PlaceContext::MutatingUse(MutatingUseContext::Store) => {
+        let source_info = body.source_info(location);
+        (source_info.span, smallvec![location])
+      }
+      PlaceContext::NonMutatingUse(NonMutatingUseContext::Inspect) => {
+        let source_info = body.source_info(location);
+        // For a statement like `let y = x + 1`, if the user selects `y`,
+        // then the only location that contains the source-map for `y` is a `FakeRead`.
+        // However, for slicing we want to give the location that actually sets `y`.
+        // So we search through the body to find the locations that assign to `y`.
+        let locations = match body.stmt_at(location) {
+          Either::Left(Statement {
+            kind:
+              StatementKind::FakeRead(box (
+                FakeReadCause::ForLet(_) | FakeReadCause::ForMatchedPlace(_),
+                _,
+              )),
+            ..
+          }) => match arg_location(*place, body) {
+            Some(arg_location) => smallvec![arg_location],
+            None => {
+              let locations = assigning_locations(body, *place);
+              if locations.len() == 0 {
+                log::warn!("FakeRead of {place:?} has no assignments");
+                return;
+              }
+              locations
+            }
+          },
+          _ => {
+            return;
+          }
+        };
+        (source_info.span, locations)
       }
       PlaceContext::NonUse(NonUseContext::VarDebugInfo)
-        if self.0.body.args_iter().any(|local| local == place.local) =>
+        if body.args_iter().any(|local| local == place.local) =>
       {
-        let source_info = self.0.body.local_decls()[place.local].source_info;
-        source_info.span
+        let source_info = body.local_decls()[place.local].source_info;
+        let location = match arg_location(*place, body) {
+          Some(arg_location) => arg_location,
+          None => location,
+        };
+        (source_info.span, smallvec![location])
       }
       _ => {
         return;
@@ -231,7 +278,7 @@ impl MirVisitor<'tcx> for MirSpanCollector<'_, '_, '_, 'tcx> {
 
     let spanned_place = MirSpannedPlace {
       place: *place,
-      location,
+      locations,
       span: span.data(),
     };
     trace!("spanned place: {spanned_place:?}");
@@ -251,10 +298,14 @@ impl MirVisitor<'tcx> for MirSpanCollector<'_, '_, '_, 'tcx> {
     self.super_statement(statement, location);
 
     if let mir::StatementKind::Assign(box (lhs, _)) = &statement.kind {
+      if lhs.ty(self.0.body.local_decls(), self.0.tcx).ty.is_unit() {
+        return;
+      }
+
       let span = try_span!(self, statement.source_info.span);
       let spanned_place = MirSpannedPlace {
         place: *lhs,
-        location,
+        locations: smallvec![location],
         span: span.data(),
       };
       trace!("spanned place (assign): {spanned_place:?}");
@@ -283,7 +334,7 @@ impl MirVisitor<'tcx> for MirSpanCollector<'_, '_, '_, 'tcx> {
     let span = try_span!(self, terminator.source_info.span);
     let spanned_place = MirSpannedPlace {
       place,
-      location,
+      locations: smallvec![location],
       span: span.data(),
     };
     trace!("spanned place (terminator): {spanned_place:?}");
@@ -291,6 +342,30 @@ impl MirVisitor<'tcx> for MirSpanCollector<'_, '_, '_, 'tcx> {
   }
 }
 
+fn assigning_locations(
+  body: &Body<'tcx>,
+  place: mir::Place<'tcx>,
+) -> SmallVec<[mir::Location; 1]> {
+  body
+    .all_locations()
+    .filter(|location| match body.stmt_at(*location) {
+      Either::Left(Statement {
+        kind: StatementKind::Assign(box (lhs, _)),
+        ..
+      })
+      | Either::Right(Terminator {
+        kind:
+          TerminatorKind::Call {
+            destination: Some((lhs, _)),
+            ..
+          },
+        ..
+      }) => *lhs == place,
+      _ => false,
+    })
+    .collect::<SmallVec<_>>()
+}
+
 pub struct Spanner<'a, 'hir, 'tcx> {
   pub hir_spans: Vec<HirSpannedNode<'hir>>,
   pub mir_spans: Vec<MirSpannedPlace<'tcx>>,
@@ -377,7 +452,7 @@ where
       self.hir_spans.iter().map(|span| (span.full, span)),
     )
     .collect::<Vec<_>>();
-    trace!("enclosing_hir={enclosing_hir:#?}");
+    // trace!("enclosing_hir={enclosing_hir:?}");
 
     if enclosing_hir.is_empty() {
       log::warn!(
